import net.mikekohn.java_grinder.CPU;
import net.mikekohn.java_grinder.IOPort0;
import net.mikekohn.java_grinder.Timer;
import net.mikekohn.java_grinder.TimerListener;
import net.mikekohn.java_grinder.Watchdog;

// 16,000,000 cycles * 0.000050 = 800 cycles
// 800 cycles = 50us
// 20 interrupts = 1ms
// 40 interrupts = 2ms
// 400 interrupts = 20ms
// 20,000 interrupts = 1s

public class ChickenPitcher implements TimerListener
{
  public static int debug = -1;
  public static int interrupt_count = 0;

  public static void waitOneSecond()
  {
    // Spinning for 20,000 interrupts is 1 second.
    interrupt_count = 0;
    while(interrupt_count != 20000);
  }

  public static void wait500ms()
  {
    // Spinning for 10,000 interrupts is 500 milliseconds.
    interrupt_count = 0;
    while(interrupt_count != 10000);
  }

  public static void main(String args)
  {
    CPU.setClock16();

    Watchdog.enable();

    IOPort0.setPinsAsOutput(0x26);
    IOPort0.setPinsLow(0xff);

    // Setup timer interrupt (aka, Java listener).
    Timer.setInterval(800, 1);
    Timer.setListener(true);

    while(true)
    {
      if ((IOPort0.getPortInputValue() & 0x1) != 1)
      {
        int data = decodeRemote();

        //if (data < 0 || data > 10) { debug = data; }
        if (data == 0x20df)
        {
          // Blink LED 3 times
          for (int i = 0; i < 3; i++)
          {
            IOPort0.setPinsHigh(0x20);
            wait500ms();
            IOPort0.setPinsLow(0x20);
            wait500ms();
          }

          // Activate solenoid
          IOPort0.setPinsHigh(0x02);
          waitOneSecond();
          IOPort0.setPinsLow(0x02);
        }
      }
    }
  }

  public static int decodeRemote()
  {
    int data;
    int bit_count;

    // Wait header ON (180 interrupts = 9ms)
    interrupt_count = 0;

    while ((IOPort0.getPortInputValue() & 0x1) == 0)
    {
      if (interrupt_count > 200) { return 1; }
    }

    if (interrupt_count < 120) { return 2; }

    // Wait header OFF (90 interrupts = 4.5ms)
    interrupt_count = 0;

    while ((IOPort0.getPortInputValue() & 0x1) == 1)
    {
      if (interrupt_count > 100) { return 3; }
    }

    if (interrupt_count < 80) { return 4; }

    data = 0;

    for (bit_count = 0; bit_count < 32; bit_count++)
    {
      // Wait data bit ON (11 interrupts = 0.562ms)
      interrupt_count = 0;

      while ((IOPort0.getPortInputValue() & 0x1) == 0)
      {
        if (interrupt_count > 15) { return 5; }
      }

      if (interrupt_count < 7) { return 6; }

      // Wait data bit OFF (11 interrupts = 0.562ms / 33 interrupts = 1.687ms))
      interrupt_count = 0;

      while ((IOPort0.getPortInputValue() & 0x1) == 1)
      {
        if (interrupt_count > 40) { return 7; }
      }

      if (interrupt_count < 7) { return 8; }

      data = data << 1;
      if (interrupt_count > 17) { data |= 1; } 

      if (bit_count == 15)
      {
        //if (data != 0x10ef) { return 9; }
        data = 0;
      }
    }

    return data;
  }

  public void timerInterrupt()
  {
    interrupt_count++;
    Watchdog.kick();
  }
}


